@@ -75,6 +75,14 @@ Nozone::Zone - record information for a bind zone
         pubkey => "MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQC1TaNgLlSyQMNWVLNLvyY/neDgaL2oqQE8T5illKqCgDtFHc8eHVAU+nlcaGmrKmDMw9dbgiGk1ocgZ56NR4ycfUHwQhvQPMUZw0cveel/8EAGoi/UyPmqfcPibytH81NFtTMAxUeM4Op8A6iHkvAMj5qLf4YRNsTkKAV"
       },
     },
+    dmarc => {
+      version => "DMARC1",
+      policy => "none",
+      subdomain_policy => "none",
+      percent => "20",
+      forensic_report => "mailto:dmarcfail@example.com",
+      aggregate_report => "mailto:dmarcagg@example.com",
+    },
     mail => {
       mx0 => {
         priority => 10,
@@ -265,7 +273,7 @@ names that are permitted to send email.
 
 The C<dkim> parameter is a hash of hash references setting
 up the DKIM records. The key for the first level hash is
-the DKIM <selector. The second level hashes contain the
+the DKIM selector. The second level hashes contain the
 following keys.
 
 The C<version> key must always be C<DKIM1>. The C<keytype>
@@ -273,6 +281,18 @@ key must be a public key algorithm name, typically 'rsa'.
 The C<service> key is a string restricting the usage.
 The C<pubkey> key is the public key.
 
+=item dmarc
+
+The C<dkim> parameter is a hash reference setting up the
+DMARC records.
+
+The C<version> key must always be C<DMARC1>. The C<policy>
+key is one of C<none>, C<quarantine> or C<reject>. The
+C<subdomain_policy> key takes the same values. The
+C<percent> key indicates how often to filter messages.
+The C<forensic_report> and C<aggregate_report> keys
+give a URI for sending reports.
+
 =item txt
 
 The C<txt> parameter is a has of arbitrary key and value
@@ -315,6 +335,7 @@ sub new {
     $self->{wildcard} = $params{wildcard} ? $params{wildcard} : undef;
     $self->{spf} = $params{spf} ? $params{spf} : undef;
     $self->{dkim} = $params{dkim} ? $params{dkim} : {};
+    $self->{dmarc} = $params{dmarc} ? $params{dmarc} : undef;
     $self->{inherits} = $params{inherits} ? $params{inherits} : undef;
 
     bless $self, $class;
@@ -703,6 +724,33 @@ sub get_spf_machines {
 }
 
 
+=item my $config = $zone->get_dmarc_config();
+
+Returns the config for the DMARC records for the domain.
+If no DMARC config is defined gainst the
+zone, returns the DMAWRC config of the parent zone.
+if there is no parent zone undefined values are
+returned indicating that no DMARC entry will be
+created.
+
+=cut
+
+sub get_dmarc_config {
+    my $self = shift;
+
+    if (defined $self->{dmarc}) {
+	return $self->{dmarc};
+    }
+
+    if ($self->{inherits}) {
+	return $self->{inherits}->get_dmarc_config();
+    }
+
+    return undef;
+}
+
+
+
 =item $zone->generate_conffile($fh, $domain, $datafile, \@masters, $verbose=0);
 
 Generate a Bind zone conf file for the domain C<$domain>
@@ -770,6 +818,7 @@ sub generate_datafile {
     $self->_generate_names($fh, $verbose);
     $self->_generate_aliases($fh, $verbose);
     $self->_generate_dkim($fh, $verbose);
+    $self->_generate_dmarc($fh, $verbose);
     $self->_generate_txt($fh, $verbose);
     $self->_generate_wildcard($fh, $verbose);
 }
@@ -1094,6 +1143,43 @@ sub _generate_dkim {
 }
 
 
+sub _generate_dmarc {
+    my $self = shift;
+    my $fh = shift;
+    my $verbose = shift;
+
+    print "    - Generate dmarc\n" if $verbose;
+
+    my $config = $self->get_dmarc_config();
+
+    return unless defined $config;
+
+    my $version = exists $config->{version} ? $config->{version} : "DMARC1";
+    my $policy = exists $config->{policy} ? $config->{policy} : "none";
+    my $subpolicy = $config->{subdomain_policy};
+    my $percent = $config->{percent};
+    my $rua = $config->{aggregate_report};
+    my $ruf = $config->{forensic_report};
+
+    my $value = "v=$version; p=$policy;";
+    if (defined $subpolicy) {
+	$value .= " sp=$subpolicy;";
+    }
+    if (defined $percent) {
+	$value .= " pct=$percent;";
+    }
+    if (defined $rua) {
+	$value .= " rua=$rua;";
+    }
+    if (defined $ruf) {
+	$value .= " ruf=$ruf;";
+    }
+    print $fh "; DMARC policy\n";
+    $self->_generate_record($fh, "_dmarc", "TXT", "", '"' . $value . '"');
+    print $fh "\n";
+}
+
+
 sub _generate_wildcard {
     my $self = shift;
     my $fh = shift;
